#########################################################################
##  LC.R
##  This is a R procedure which calculates the LC test
##  for parameter instability in linear models as reported
##  in Hansen (1992) Journal of Policy Modeling.
##  
##  Bruce E. Hansen
##  Department of Economics
##  Social Science Building
##  University of Wisconsin
##  Madison, WI 53706-1393
##  behansen@wisc.edu
##  http://www.ssc.wisc.edu/~bhansen/
##  
##  
##  The format for the command is
##  
##  out <- lc(dat,datname)
##  
##  The matrix "dat" is a nx(k+1) matrix, with the dependent
##  (y) variable in the first colume, and the independent (x)
##  variables in the remaining columes.  If a constant is desired
##  in the regression, it should be included in the dat matrix.
##  The vector "datname" is a (k+1)x1 list of variable names,
##  for the columes of "dat".
##  
##  The output is printed to the screen.
##  
#########################################################################

lc <- function(dat,datname){

n <- nrow(dat)
k <- ncol(dat)-1
y <- as.matrix(dat[,1])
x <- dat[,2:(k+1)]
m <- solve(t(x)%*%x) 
b <- m%*%(t(x)%*%y)
e <- y-x%*%b
xe <- x*(e%*%matrix(1,1,ncol(x)))
e2 <- e^2
sig2 <- mean(e2)
e2m <- e2-sig2
se <- rbind(as.matrix(sqrt(diag(m%*%(t(xe)%*%xe)%*%m))),sqrt(mean(e2m^2)/n))
r2 <- 1-sum(e2)/sum((y-mean(y))^2)
f <- cbind(xe,e2m)
s <- matrix(0,nrow(f),ncol(f))
for (j in 1:ncol(f)) s[,j] <- cumsum(f[,j]) 
v <- t(f)%*%f
lj <- sum(s*(s%*%solve(v)))/n
li <- as.matrix(colSums(s^2)/diag(v)/n)
datnames <- rbind(as.matrix(datname[2:(k+1)]),"Variance")

cat ("Dependent Variable = ", datname[1],"\n")
cat ("\n")
cat ("Variable   ","Estimate   ","St Error   ","Lc        ","\n")
cat ("____________________________________________________","\n")
datnames <- format(datnames,digits=4)
Tb <- format(rbind(b,sig2),digits=4)
Tse <- format(se,digits=4)
Tli <- format(li,digits=4)
for (j in 1:nrow(Tb)) cat (datnames[j]," ",Tb[j]," ",Tse[j]," ",Tli[j],"\n")
cat ("____________________________________________________","\n")
cat ("\n")
cat ("\n")
cat ("Joint LC      ", lj, "\n")
cat ("R-squared     ", r2, "\n")
cat ("\n")
cat ("\n")
cat ("************************************************************","\n")
cat ("\n")
cat ("\n")
}


